package main

import (
	"encoding/json"
	"errors"
	"fmt"
	"io/ioutil"
	"log"
	"net/http"
	"net/url"
	"strings"

	"github.com/mattermost/mattermost-server/v6/plugin"
	"github.com/mattermost/mattermost-server/v6/model"
)

type Plugin struct {
	plugin.MattermostPlugin
	configuration *configuration
}

type configuration struct {
	APIKey  string
	BaseURL string
}

func (p *Plugin) OnActivate() error {
	// Log configuration on activation
	config := p.getConfiguration()
	if config == nil {
		log.Println("Error: plugin configuration is nil on activation")
	} else {
		log.Printf("Plugin activated with config - APIKey: %s, BaseURL: %s", 
			config.APIKey, config.BaseURL)
	}

	p.API.RegisterCommand(&model.Command{
		Trigger:          "profanityblocker",
		AutoComplete:     true,
		AutoCompleteDesc: "Manage the Profanity Blocker plugin.",
		AutoCompleteHint: "[command]",
	})
	return nil
}

func (p *Plugin) filterProfanity(message string) (string, error) {
	config := p.getConfiguration()
	if config == nil {
		return message, errors.New("configuration is nil")
	}

	// Log request details
	log.Printf("Sending request to %s with API key %s", config.BaseURL, config.APIKey)

	apiURL := config.BaseURL + "filter/sendstring_api"
	
	// Create form data
	formData := url.Values{}
	formData.Set("requestid", config.APIKey)
	formData.Set("origstring", message)
	formData.Set("url", "mattermost.com")
	formData.Set("phone", "false")
	formData.Set("email", "false")

	// Create request with form data
	req, err := http.NewRequest("POST", apiURL, strings.NewReader(formData.Encode()))
	if err != nil {
		return message, err
	}
	req.Header.Set("Content-Type", "application/x-www-form-urlencoded")

	client := &http.Client{}
	resp, err := client.Do(req)
	if err != nil {
		log.Printf("Error making request: %v", err)
		return message, err
	}
	defer resp.Body.Close()

	responseBody, _ := ioutil.ReadAll(resp.Body)
	log.Printf("API Response: %s", string(responseBody))

	// Parse JSON response
	var result map[string]interface{}
	if err := json.Unmarshal(responseBody, &result); err != nil {
		log.Printf("Error parsing response: %v", err)
		return message, err
	}

	if newdata, ok := result["newdata"].(string); ok {
		return newdata, nil
	}

	// Check for error codes
	if errornum, ok := result["errornum"].(float64); ok {
		log.Printf("API returned error code: %v", errornum)
		return message, fmt.Errorf("API error code: %v", errornum)
	}

	return message, nil
} 

func (p *Plugin) MessageWillBePosted(c *plugin.Context, post *model.Post) (*model.Post, string) {
	if post.IsSystemMessage() {
		return post, ""
	}

	filteredContent, err := p.filterProfanity(post.Message)
	if err != nil {
		log.Printf("Error filtering profanity: %v", err)
		// Return error message to user
		return nil, fmt.Sprintf("Error processing message: %v", err)
	}

	// Check if the original message is different from the filtered message
	if post.Message != filteredContent && filteredContent == "Profanity detected" {
		// Log the deletion
		log.Printf("Profanity detected. Deleting message: %s", post.Message)
		
		// Send ephemeral feedback to the user
		ephemeralPost := &model.Post{
			ChannelId: post.ChannelId,
			Message:   "🚫 Your message was blocked because it contains inappropriate content",
			Type:      "system_ephemeral",
		}
		p.API.SendEphemeralPost(post.UserId, ephemeralPost)
		
		// Return rejection message
		return nil, "Message blocked due to inappropriate content"
	}

	post.Message = filteredContent
	return post, ""
}

func main() {
	plugin.ClientMain(&Plugin{})
} 